// ============================================================================
// zipstream, C++ iostream classes wrapping the zlib compression library.
// Copyright (C) 2007  Thomas Sailer
// Copyright (C) 2001  Deepak Bandyopadhyay, Lutz Kettner
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// ============================================================================
//
// File          : zipstream.C
// Revision      : $Revision: 1.1 $
// Revision_date : $Date: 2005/05/25 18:20:56 $
// Author(s)     : Thomas Sailer, Deepak Bandyopadhyay, Lutz Kettner
// 
// Standard streambuf implementation following Nicolai Josuttis, "The 
// Standard C++ Library".
// ============================================================================

#include <iostream>
#include <string.h>		// for memcpy
#include "zipstream.hpp"
#include <zip.h>
#include <unzip.h>

namespace zfstream
{

// ----------------------------------------------------------------------------
// Internal classes to implement zipstream. See header file for user classes.
// ----------------------------------------------------------------------------

#define CASESENSITIVITY (0)

// --------------------------------------
// class zipstreambuf:
// --------------------------------------

	zipstreambuf *zipstreambuf::open( const char *name, int open_mode, const char *fname, const char *passwd )
	{
		if ( is_open(  ) )
			return ( zipstreambuf * ) 0;
		mode = open_mode;
		// no append nor read/write mode
		if ( ( mode & std::ios::ate ) || ( mode & std::ios::app ) || ( ( mode & std::ios::in ) && ( mode & std::ios::out ) ) )
			return ( zipstreambuf * ) 0;
		if (mode & std::ios::in) {
			m_file = (void *)unzOpen(name);
			if (!m_file)
				return 0;
			int err = UNZ_OK;
			if (err == UNZ_OK) {
				if (fname)
					err = unzLocateFile((unzFile)m_file, fname, CASESENSITIVITY);
				else
					err = unzGoToFirstFile((unzFile)m_file);
			}
			if (err == UNZ_OK) {
				if (passwd)
					err = unzOpenCurrentFilePassword((unzFile)m_file, passwd);
				else
					err = unzOpenCurrentFile((unzFile)m_file);
				if (err != UNZ_OK)
					unzCloseCurrentFile((unzFile)m_file);
			}
			if (err != UNZ_OK) {
				unzClose((unzFile)m_file);
				m_file = 0;
				return 0;
			}
		} else if (mode & std::ios::out) {
			m_file = (void *)zipOpen(name, APPEND_STATUS_CREATE);
			if (!m_file)
				return 0;
			int err = ZIP_OK;
			if (err == ZIP_OK)
				err = zipOpenNewFileInZip3((zipFile)m_file, fname, NULL, NULL, 0, NULL, 0, NULL, (m_zlevel < 0) ? 0 : Z_DEFLATED, m_zlevel, -MAX_WBITS, DEF_MEM_LEVEL, Z_DEFAULT_STRATEGY, 0, 0, 0);
			if (err != ZIP_OK) {
				zipClose((zipFile)m_file, NULL);
				m_file = 0;
				return 0;
			}
		} else
			return 0;
		opened = 1;
		return this;
	}

	zipstreambuf *zipstreambuf::close()
	{
		if ( is_open(  ) )
		{
			sync(  );
			opened = 0;
			if (mode & std::ios::in) {
				unzCloseCurrentFile((unzFile)m_file);
				unzClose((unzFile)m_file);
			}
			if (mode & std::ios::out) {
				zipCloseFileInZip((zipFile)m_file);
				zipClose((zipFile)m_file, NULL);
			}
			m_file = 0;
			return this;
		}
		return ( zipstreambuf * ) 0;
	}

	int zipstreambuf::underflow(  )
	{			// used for input buffer only
		if ( gptr(  ) && ( gptr(  ) < egptr(  ) ) )
			return *reinterpret_cast < unsigned char *>( gptr(  ) );

		if ( !( mode & std::ios::in ) || !opened )
			return EOF;
		// Josuttis' implementation of inbuf
		int n_putback = gptr(  ) - eback(  );
		if ( n_putback > 4 )
			n_putback = 4;
		memcpy( buffer + ( 4 - n_putback ), gptr(  ) - n_putback, n_putback );

		int num = unzReadCurrentFile((unzFile)m_file, buffer + 4, bufferSize - 4);
		if ( num <= 0 )	// ERROR or EOF
			return EOF;

		// reset buffer pointers
		setg( buffer + ( 4 - n_putback ),	// beginning of putback area
		      buffer + 4,	// read position
		      buffer + 4 + num );	// end of buffer

		// return next character
		return *reinterpret_cast < unsigned char *>( gptr(  ) );
	}

	int zipstreambuf::flush_buffer(  )
	{
		// Separate the writing of the buffer from overflow() and
		// sync() operation.
		int w = pptr(  ) - pbase(  );
		if ( zipWriteInFileInZip((zipFile)m_file, pbase(), w) != ZIP_OK )
			return EOF;
		pbump( -w );
		return w;
	}

	int zipstreambuf::overflow( int c )
	{			// used for output buffer only
		if ( !( mode & std::ios::out ) || !opened )
			return EOF;
		if ( c != EOF )
		{
			*pptr(  ) = c;
			pbump( 1 );
		}
		if ( flush_buffer(  ) == EOF )
			return EOF;
		return c;
	}

	int zipstreambuf::sync(  )
	{
		// Changed to use flush_buffer() instead of overflow( EOF)
		// which caused improper behavior with std::endl and flush(),
		// bug reported by Vincent Ricard.
		if ( pptr(  ) && pptr(  ) > pbase(  ) )
		{
			if ( flush_buffer(  ) == EOF )
				return -1;
		}
		return 0;
	}

// --------------------------------------
// class zipstreambase:
// --------------------------------------

	zipstreambase::zipstreambase( const char *name, int mode )
	{
		init( &buf );
		open( name, mode );
	}

	zipstreambase::~zipstreambase(  )
	{
		buf.close(  );
	}

	void zipstreambase::open( const char *name, int open_mode )
	{
		if ( !buf.open( name, open_mode ) )
			clear( rdstate(  ) | std::ios::badbit );
	}

	void zipstreambase::close(  )
	{
		if ( buf.is_open(  ) )
			if ( !buf.close(  ) )
				clear( rdstate(  ) | std::ios::badbit );
	}

}				// namespace

// ============================================================================
// EOF //
