// ============================================================================
// zipstream, C++ iostream classes wrapping the zlib compression library.
// Copyright (C) 2007  Thomas Sailer
// Copyright (C) 2001  Deepak Bandyopadhyay, Lutz Kettner
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// ============================================================================
//
// File          : zipstream.h
// Revision      : $Revision: 1.1 $
// Revision_date : $Date: 2005/05/25 18:20:56 $
// Author(s)     : Thomas Sailer, Deepak Bandyopadhyay, Lutz Kettner
// 
// Standard streambuf implementation following Nicolai Josuttis, "The 
// Standard C++ Library".
// ============================================================================

#ifndef zfstream_ZIPSTREAM_H_INCLUDED
#define zfstream_ZIPSTREAM_H_INCLUDED 1


// standard C++ with new header file names and std:: namespace
#include <iostream>
#include <fstream>
#include <zlib.h>

namespace zfstream
{

// ----------------------------------------------------------------------------
// Internal classes to implement zipstream. See below for user classes.
// ----------------------------------------------------------------------------
	class zipstreambuf:public std::streambuf
	{
	      private:
		static const int bufferSize = 47 + 256;	// size of data buff
		// totals 512 bytes under g++ for izipstream at the end.

		char buffer[bufferSize];	// data buffer
		char opened;	// open/close state of stream
		int mode;	// I/O mode

		int flush_buffer();
	      public:
                zipstreambuf():opened( 0 ), m_zlevel(-1)
		{
			setp( buffer, buffer + ( bufferSize - 1 ) );
			setg( buffer + 4,	// beginning of putback area
			      buffer + 4,	// read position
			      buffer + 4 );	// end position      
			// ASSERT: both input & output capabilities will not be used together
		}
		int is_open()
		{
			return opened;
		}
		zipstreambuf *open( const char *name, int open_mode, const char *fname = 0, const char *passwd = 0 );
		zipstreambuf *close();
		~zipstreambuf()
		{
			close();
		}

		virtual int overflow( int c = EOF );
		virtual int underflow();
		virtual int sync();

                void zlevel( int z ) { m_zlevel = ( z<0 ? -1 : (z>9?9:z) ) ; }
                int zlevel() const  { return m_zlevel; }
        private:
                int m_zlevel;
		void *m_file;
	};

        /**
           Base stream class inherited by ozipstream/izipstream. Any
           given instance may not be used for both input and output.

           Uses gzlib compression, and is compatible with files compressed
           using gzip, or any number of other tools which support gzlib.

        */
	class zipstreambase:virtual public std::ios
	{
        private:
                int m_zlevel;
	      protected:
		zipstreambuf buf;
	      public:
		zipstreambase()
		{
			init( &buf );
		}
		zipstreambase( const char *name, int open_mode );
		~zipstreambase();
		void open( const char *name, int open_mode );
		void close();

                void zlevel( int z ) { buf.zlevel(z); }
                int zlevel() const  { return buf.zlevel(); }

		zipstreambuf *rdbuf()
		{
			return &buf;
		}
	};


        /**
           An input stream which decompresses it's input. Used
           identically to a std::ifstream.

<pre>
izipstream ifs( "myfile.gz" );
</pre>
        */
	class izipstream:public zipstreambase, public std::istream
	{
	      public:
		izipstream():std::istream( &buf )
		{
		}
	      izipstream( const char *name, int open_mode = std::ios::in ):zipstreambase( name, open_mode ), std::istream( &buf )
		{
		}
		zipstreambuf *rdbuf()
		{
			return zipstreambase::rdbuf();
		}
		void open( const char *name, int open_mode = std::ios::in )
		{
			zipstreambase::open( name, open_mode );
		}
	};

        /**
           An output stream which compresses it's output. Used identically to a std::ofstream:

<pre>
ozipstream of( "myfile.gz" );
of << "hello, world!" << std::endl;
</pre>
        */
	class ozipstream:public zipstreambase, public std::ostream
	{
	      public:
		ozipstream():std::ostream( &buf )
		{
		}
	      ozipstream( const char *name, int mode = std::ios::out ):zipstreambase( name, mode ), std::ostream( &buf )
		{
		}
		zipstreambuf *rdbuf()
		{
			return zipstreambase::rdbuf();
		}
		void open( const char *name, int open_mode = std::ios::out )
		{
			zipstreambase::open( name, open_mode );
		}
	};

}				// namespace

#endif // zfstream_ZIPSTREAM_H_INCLUDED
// ============================================================================
// EOF //
